#! /usr/bin/env python
# -*- encoding: utf-8 -*-


# gimp-help-2 --XML language separator tool
# Copyright (C) 2006 Joao S. O. Bueno Calligaris
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.
#



import sys

#download elementtree from http://effbot.org/downloads/
#it is set to be included in the core Python as of version 2.5
import elementtree.ElementTree as ET
from copy import copy, deepcopy
import os.path
from os import mkdir

IDTAG = "xml_id"
HASHTAG = "xml_hash"
CHANGEDTAG = "xml_changed"



work_dir = "./working_translation"

version = "0.91"
#TODO (v.0.9): properly formating the final gimp-help xml files
#that is: Adding a DTD header, and doing line wrapping and identation


class Stub(object):
    def __init__(self, node=None):
        self.node = node
        self.children_langs = {}
    def getchildren (self):
        return [self.node]


def recurse_anotate (parent, document):
    """
    Since in elementTree tehre sis no way to get  a reference to a node ? siblings or parent, this takes care of it.
    """
    document.parent = parent
    siblings = parent.getchildren ()
    self_index = siblings.index (document)
    if self_index != 0:
        document.previous_sibling = siblings[self_index - 1]
    else:
        document.previous_sibling = None

    if self_index + 1 < len ( siblings):
        document.next_sibling = siblings[self_index + 1]
    else:
        document.next_sibling = None
    for child in document:
        recurse_anotate (document, child)

def make_dir (path):
    if path == "":
        return
    while not os.path.isdir (path):
        make_dir (os.path.dirname(path))
        mkdir (path)

def create_new_ids (document):
    idcount = 1
    for node in document.getiterator ():
        node_hash = str (ET.tostring(node, "utf-8").__hash__())
        node.attrib [IDTAG] = str (idcount)
        node.attrib [HASHTAG] = node_hash
        idcount += 1

def parse_hashes (document):
    all_hashes = {}
    for node in document.getiterator ():
        all_hashes [node.attrib[HASHTAG]] = node
    return all_hashes

def parse_ids (document):
    all_ids = {}
    for node in document.getiterator ():
        all_ids [node.attrib[IDTAG]] = node
    return all_ids

def get_node_by_id (document, id):
    for node in document.getiterator ():
        if node.attrib.has_key (IDTAG) and \
           node.attrib[IDTAG] == id:
            return node
    return None
def get_all_langs (document):
    """Rerieve all languages in use  a document"""
    langs = {}
    for node in document.get_iterator():
        if node.attrib.has_key ("lang"):
            if ";" in node.attrib["lang"]:
                local_langs = node.attrib["lang"].split(";")
            else:
                local_langs = [node.attrib["lang"]]
            for local_lang in local_langs:
                langs[local_lang] = None
    return langs.keys ()

def node_replace (parent_node, old_node, new_node):
    parent_node.insert (parent_node.getchildren().index(old_node), new_node)
    parent_node.remove (old_node)

def node_insert_after (parent_node, old_node, new_node):
    #this is not working:
    #parent_node.insert (parent_node.getchildren().index(old_node) + 1, new_node)
    for i, node in enumerate (parent_node.getchildren()):
        if node.attrib[IDTAG] == old_node.attrib[IDTAG]:
            break
    else:
        sys.stderr ("Error placing new tag - putting at the end of parent tag")
    parent_node.insert (i + 1, new_node)



def create_ids (path, document):
    try:
        helper_document = ET.parse (path).getroot()
    except:
        create_new_ids (document)
        return
    #Here it will check witch nodes in any of the selected languages had
    # changed since the file was last merged back, and add a "fuzzy"
    # xml attribute to tehm.

    #this may take a while.

    #nodes with the same content may clash and give false positives
    #of having changed. That will not be a big deal.
    all_hashes = parse_hashes (helper_document)
    idcount = 1
    for node in document.getiterator ():
        node_hash = str (ET.tostring(node, "utf-8").__hash__())
        if not all_hashes.has_key (node_hash):
            node.attrib [CHANGEDTAG] = "True"
        node.attrib [IDTAG] = str (idcount)
        node.attrib [HASHTAG] = node_hash
        idcount += 1

def remove_ids (document):
    for node in document.getiterator ():
        for attrib in (CHANGEDTAG, HASHTAG, IDTAG):
            if node.attrib.has_key (attrib):
                del node.attrib [attrib]


def remove_hashes (document):
    for node in document.getiterator ():
        if node.attrib.has_key (HASHTAG):
            del node.attrib [HASHTAG]
        if (node.attrib.has_key (IDTAG) and
            not node.attrib.has_key ("lang") and
            not node.getchildren()
            ):
            del node.attrib [IDTAG]

def create_node_order_helper (document):
    counter = 0
    for node in document.getiterator ():
        node.xml_helper_order = counter
        counter += 1

def _sort_nodes (node_list):
    reindex = {}
    for i, node in enumerate (node_list):
        reindex [node.xml_helper_order] = i
    new_list = []
    sorted_keys = reindex.keys()[:]
    sorted_keys.sort()
    for key in sorted_keys:
        new_list.append (node_list[reindex[key]])
    return new_list



def recurse_rip (parent, element, all_langs):
    element.children_langs = {}
    if not element.attrib.has_key ("lang"):
        for child in element.getchildren()[:]:
            recurse_rip (element, child, all_langs)
    else:
        elem_langs = element.attrib["lang"].split (";")
        first_lang_found = False
        for elem_lang in  elem_langs:
            if elem_lang in all_langs:
                try:
                    parent.children_langs[elem_lang] += 1
                except KeyError:
                    parent.children_langs[elem_lang] = 1
                if not first_lang_found:
                    for child in element.getchildren()[:]:
                        recurse_rip (element, child, all_langs)
                    first_lang_found = True
        if not first_lang_found:
            if element.tail.isspace():
                element.tail = ""
            #print ET.tostring (element, "utf-8")
            parent.remove (element)
    if element.children_langs:
        pass
    del element.children_langs

def count_children_lang (node):
    count = 0
    first = True
    for child in node.getiterator ():
        if first:
            first = False
            continue
        if node.attrib.has_key("lang"):
            count += 1
    return count

def merge_docs (helper_document, translated_document, merge_langs):
    recurse_anotate (Stub(helper_document), helper_document)
    recurse_anotate (Stub(translated_document), translated_document)
    all_ids = parse_ids (helper_document)
    for node in translated_document.getiterator ():
        #this extra attribute is retrieved in the sorting
        #by lang attributes routine to assure the same
        #document order of the nodes with the same lang.
        if not node.attrib.has_key ("lang"):
            continue
        if (len (node.attrib["lang"].split(";") ) > 1 and
            len (node.getchildren ())  and
            count_children_lang (node) != 0 ):
            continue

        found_lang = False
        elem_langs = node.attrib["lang"].split (";")
        for elem_lang in  elem_langs:
            if elem_lang in merge_langs:
                found_lang = True
                break
        if not found_lang:
            continue
        #if it got here, current node in document is to be
        #merged into helper_document
        if node.attrib.has_key (IDTAG):
            node_to_be_replaced = all_ids [node.attrib[IDTAG]]
            node_replace (node_to_be_replaced.parent, node_to_be_replaced, node)
        else:
            #find the nearest node with an idtag.
            address_node = node
            found = False
            while address_node.previous_sibling != None:
                address_node = address_node.previous_sibling
                if address_node.attrib.has_key (IDTAG):
                    prev_id = address_node.attrib[IDTAG]
                    where_to_insert = all_ids [prev_id]
                    found = True
                    break
            if not found:
                prev_id = node.parent.attrib[IDTAG]
                where_to_insert = all_ids [prev_id].getchildren()[0]
            node_insert_after (where_to_insert.parent, where_to_insert, node)
    return helper_document

def _format_xml_internal (child_langs,
                          node,
                          nodes_by_lang
                         ):
            if not len (child_langs):
                return
            index = node.getchildren().index (
                            nodes_by_lang [child_langs[0]] [0] )
            for remove_1 in nodes_by_lang:
                for remove_2 in nodes_by_lang[remove_1]:
                    node.remove (remove_2)
            child_langs.sort ()
            #restore original order of the text - it may have mixed up
            #after merging and sorting languages
            for lang in nodes_by_lang:
                nodes_by_lang[lang] = _sort_nodes (nodes_by_lang[lang])

            for lang in child_langs:
                node.insert (index, nodes_by_lang [lang].pop(0))
                index += 1

def format_xml (node):
    """Recursively sort the chidren tags by lang attrib"""
    group = None
    child_langs = []
    nodes_by_lang = {}
    for child in node.getchildren():
        do_grandchildren = True
        if child.tag != group:
            _format_xml_internal (child_langs, node, nodes_by_lang)
            child_langs = []
            nodes_by_lang = {}
            group = child.tag
        if child.attrib.has_key ("lang"):

            lang = child.attrib["lang"]
            if len (lang.split (";")) > 1:
                langs = lang.split(";")
                langs.sort ()
                lang = ";".join (langs)
                child.attrib["lang"] = lang
            else:
                do_grandchildren = False
            child_langs.append (lang)
            try:
                nodes_by_lang[lang].append (child)
            except:
                nodes_by_lang[lang] = [child]
        if do_grandchildren:
            format_xml (child)
    else: #at the end of this nodes' children
        _format_xml_internal (child_langs, node, nodes_by_lang)

def _create_missing_tags_internal (node,
                                   group_langs,
                                   base_lang
                                  ):

        number_base_nodes = len (group_langs [base_lang])
        if not number_base_nodes:
            return

        for lang in group_langs:
            if lang == base_lang:
                continue
            if len (group_langs [lang]) < number_base_nodes:
                if len (group_langs [lang]):
                    index = node.getchildren().index (group_langs
                                                    [lang][-1])
                else:
                    index = node.getchildren().index (group_langs
                                                [base_lang][-1]) + 1
                    #If a target language has less tags than
                    #the base language in  a tag group,
                    #the later nodes of base lang will be cloned
                    #as the later clones of target lang, regardless
                    #of their content.
                for copying_node in group_langs[base_lang] [
                    len (group_langs[lang]):]:
                    new_node = deepcopy (copying_node)
                    new_node.attrib ["lang"] = lang
                    if "no-clone-content" in options:
                        new_node.clear()
                    node.insert (index, new_node)
                    index += 1


def create_missing_tags (node, all_langs):
    """
        when separating  a file into fewer languages for translation,
        create new tags for the target languages if the first language
        on the language list has more tags in each group
        of equal tags among siblings than the other languages
    """
    base_lang = all_langs[0]
    langs = all_langs[1:]
    group = None
    child_langs = []
    group_langs = {}
    for lang in all_langs:
        group_langs[lang] =  []

    for child in node.getchildren():
        do_grandchildren = True
        if child.tag != group:
            _create_missing_tags_internal (node, group_langs, base_lang)

            child_langs = []
            group_langs = {}
            for lang in all_langs:
                group_langs[lang] =  []
            group = child.tag
        if child.attrib.has_key ("lang"):
            lang = child.attrib["lang"]
            if len (lang.split (";")) > 1:
                local_langs = lang.split (";")
                found = False
                for local_lang in local_langs:
                    if local_lang in all_langs:
                        found = True
                        break
                if found:
                    local_missing = []
                    for global_lang in all_langs:
                        if global_lang not in local_langs:
                            local_missing.append (global_lang)
                    if len (local_missing):
                        local_langs += local_missing
                        local_langs_txt = ";".join (local_langs)
                        child.attrib["lang"] = local_langs_txt
            else:
                if lang in all_langs:
                    child_langs.append (lang)
                    group_langs [lang].append (child)

                do_grandchildren = False
        if do_grandchildren:
            create_missing_tags (child, all_langs)
    else: #at the end of the siblings
        _create_missing_tags_internal (node, group_langs, base_lang)


def rip (file, langs):
    try:
        all_document = ET.parse (file)
    except:
        sys.stderr.write ("Error parsing file %s. Skipped\n")
        return
    document = all_document.getroot ()
    current_langs = copy (langs)

    #TODO:
    #compare_files (document)

    output_helper_name =  os.path.join (work_dir,
                                        os.path.dirname (file),
                                        ".helper",
                                        os.path.basename (file)
                                       )
    make_dir (os.path.dirname (output_helper_name))

    if not "no-create-tags" in options:
        create_missing_tags (document, langs)

    create_ids (output_helper_name, document)

    all_document.write (output_helper_name, "utf-8")
    recurse_rip (Stub (), document, langs)

    remove_hashes (document)
    output_file_name = os.path.join (work_dir, file)
    all_document.write (output_file_name,  "utf-8")


def merge (file, langs):
    input_helper_name =  os.path.join (work_dir,
                              os.path.dirname (file),
                              ".helper",
                               os.path.basename (file)
                                       )
    input_file_name = os.path.join (work_dir, file)

    helper_document = ET.parse (input_helper_name).getroot()
    translated_document = ET.parse (input_file_name).getroot()

    merge_docs (helper_document, translated_document, langs)

    document = helper_document
    remove_ids (document)
    if not "no-format" in options:
        create_node_order_helper (document)
        format_xml (document)

    all_document = ET.ElementTree (document)
    all_document.write (file, "utf-8")

    #write copy with IDs and hash, so that when this file is
    #next splited, changed tags get marked with "changed"
    create_new_ids (document)

    all_document.write (input_helper_name, "utf-8")

    if not "no-delete-rip" in options:
        os.unlink (input_file_name)

def format (file):
    all_document = ET.parse (file)
    document = all_document.getroot()
    create_node_order_helper (document)
    format_xml (document)
    all_document.write (file, "utf-8")
    #print ET.tostring (document,"utf-8")
    #TODO:

    #format_textual_xml (file)

    #this function should take care of
    #DTD headers, indentation and line length
    #reading and writing to the XML file as a text file



def rip_files (in_files, langs):
    for file in in_files:
        rip (file, langs)

def merge_files (in_files, langs):
    for file in in_files:
        merge (file, langs)

def format_files (in_files):
    for file in in_files:
        format (file)

def short_help():
    global work_dir
    sys.stderr.write ( """\
\nExamples of use:
    ./tools/rip_lang.py --rip --langs=en,fr src/gimp.xml
    ./tools/rip_lang.py --merge --langs=fr src/gimp.xml

Execute  xml_helper.py --help to see the full help text
\n""")

def help():
    global work_dir
    sys.stderr.write ( """\
Usage:
rip_lang.py --rip|--merge|--format  [--no-format] [--no-clone-content]
[--no-create-tags]
--lang=xx,yy,zz file1, file2 ...filen

This script Will create versions of these XML files at %s directory
containing only the tags that include the selected languages
 (and creating empty tags if there is a tag for one
language and not for the others).

The tags that already exist will receive a xmlid attribute -
keep it unchanged, or the merger script won't work.

It will also create a .helper directory in that same directory, where
it will store a version of the original XML with some helper tags.
Do not remove these files, or the merger script won't be able to
merge your modifications back into the main XML.

Operation modes:

--rip:
    will generate a new XML document containing only the
    desired languages (see --lang). XML created this way
    have an extra xmlid attrib in all tags. This is used when
    integrating back into  the main XML. You may write more tags
    for a given language, and they will be integrated - leave these
    without a "xmlid" attrib. All new tags must have a lang attrib
    or be inside a tag with alang attrib.


--merge:
    will integrate back a working copy of a XML file into
    the main XML structure. The hidden version in .helper
    dir must exist. Merged files are formated as for the
    --format mode, unless stated otherwise.
    N.B. the paths to the files passed to the script are the paths
    on main src tree, not the in the working dir.

--format:
    will format target xml files as per the guidelines in the
    HACKING file.
    NOT FULLY IMPLEMENTED - At this time,
    this will just sort the tags by the lang attrib.

--help:
    Displays this help and exits

--version:
    Displays the script version and exits


Options:

    --langs:
        mandatory for the --rip and --merge modes - the selected languages
        that will be kept on the working version (ruip) or
        merged back in the main XML (merge).
        The oprder matters in (rip) mode - the first language
        specified is the one that will be clonned in newly created tags
        for the other languages if they don't exist.
        Usage: --langs=xx,yy,zz, ... where xx, yy and zz are language
        codes

    --no-create-tags:
        in --rip mode, doesn't create tags that do not exist for a given
        language, when compared to the first language specified

    --no-clone-content:
        create new language tags, but empty tags, instead of copying
        the content of the first language specified. NB - tags created in
        this way are self closed ( <tag  />  ), by the xml generator.
        Dont forget to change the format to <tag> </tag>  when adding
        content.
    --no-format:
        passed to merge mode - doesn't run the formatting routines
        after merging the xml file.

    --no-delete-rip:
        To avoid conflicts, the working XML version is deleted when merging
        is successfull. If passing this option, the file is preserved (the
        xmlids change when merging, so it is basically impossible to use
        this script to merge a XML a second time.

General Notes:

If a version of the XML being split already exists in the
.helper dir, it will mark changed tags since the last split with
a 'changed=""' XML attribute. This way translations can be kept in sink
without further resource to Harry Potter style magic.

This script will not normally  supress existing XML tags. If
a tag is deleted in the working file, it's copy in the  .helper file
will be used.

Workflow example :

cd gimp-help-2
./tools/rip_lang.py --rip --langs=en,pt_BR src/gimp.xml
cd %s/src
<editor_of_choice> gimp.xml
<translate>
<save><exit>
cd ../..
./tools/rip_lang.py --merge --langs=pt_BR src/gimp.xml
\n

(C) 2006 João S. O. Bueno Calligaris
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
\n""" %(work_dir, work_dir))


if __name__=="__main__":
    langs = None
    in_files = []
    options = []
    for option in sys.argv[1:]:
       if option[0:8]=="--langs=":
           langs = option[8:].split(",")
       elif option == "--help":
           help ()
           sys.exit(0)
       elif option == "--version":
           print "xml_helper version %s\nby João S. O. Bueno Calligaris (c) 2006" % version
           sys.exit(0)
       elif option[:2] != "--":
           in_files.append (option)
       else:
           options.append (option[2:])
    if  not in_files:
        short_help()
        sys.exit (1)
    if "rip" in options and langs:
        rip_files (in_files, langs)
    elif "merge" in options and langs:
        merge_files (in_files, langs)
    elif "format" in options:
        format_files (in_files)
    else:
        short_help()
        print options, langs
        sys.exit (1)


# -*- coding: utf-8 -*-
# Copyright (c) 2004, 2005, 2006 Danilo Segan <danilo@gnome.org>.
#
# This file is part of xml2po.
#
# xml2po is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# xml2po is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with xml2po; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Adapted for the GIMP Manual's build system 2009-12-14
# (c) 2009 The GIMP Documenation Team
#

import re
import libxml2
import os
import sys
try:
    # Hashlib is new in Python 2.5
    from hashlib import md5 as md5_new
except ImportError:
    from md5 import new as md5_new

from docbook import docbookXmlMode

class gimphelpXmlMode(docbookXmlMode):
    """Class for special handling of gimp-help DocBook document types.

    It sets lang attribute on article elements, and adds translators
    to articleinfo/copyright."""
    def __init__(self):
        try:
            super(gimphelpXmlMode, self).__init__()
        except TypeError:
            self.lists = ['itemizedlist', 'orderedlist', 'variablelist',
                          'segmentedlist', 'simplelist', 'calloutlist', 'varlistentry' ]
            self.objects = [ 'figure', 'textobject', 'imageobject', 'mediaobject',
                             'screenshot' ]

    def getTreatedAttributes(self):
        "Return array of tag attributes which content is to be translated"
        return ['xreflabel']

    def _output_images(self, node, msg):
        assert node
        if node.type == 'element' and node.name == 'imagedata':
            # Use .fileref to construct new message
            attr = node.prop("fileref")
            if attr:
                assert attr.startswith("images/")
                origimagepath = attr
                for subdir in ("C", "commmon"):
                    imagepath = origimagepath.replace("/", "/%s/" % subdir, 1)
                    if os.path.exists(imagepath):
                        hash = self._md5_for_file(imagepath)
                        break
                else:
                    hash = "THIS FILE DOESN'T EXIST"
                    sys.stderr.write("Warning: image file '%s' not found.\n" %
                                     origimagepath)
                msg.outputMessage(
                    "@@image: '%s'; md5=%s" % (origimagepath, hash),
                    node.lineNo(),
                    "When image changes, this message will be marked fuzzy "
                    "or untranslated for you.\nIt doesn't matter what you "
                    "translate it to: it's not used at all.")
        elif node and node.children:
            child = node.children
            while child:
                self._output_images(child,msg)
                child = child.next

    def preProcessXml(self, doc, msg):
        """Add additional messages of interest here."""
        root = doc.getRootElement()
        self._output_images(root,msg)

# Perform some tests when ran standalone
if __name__ == '__main__':
    test = gimphelpXmlMode()
    print "Ignored tags       : " + repr(test.getIgnoredTags())
    print "Final tags         : " + repr(test.getFinalTags())
    print "Space-preserve tags: " + repr(test.getSpacePreserveTags())

    print "Credits from string: '%s'" % test.getStringForTranslators()
    print "Explanation for credits:\n\t'%s'" % test.getCommentForTranslators()




#!/usr/bin/env python
# -*- coding: latin1 -*-
"""
  Validate image file references

  Validates image file references in DocBook XML files and
  finds orphaned images files.
"""

# gimp-help-2 -- validate_references.py
#
# Copyright (C) 2006, 2007 Róman Joost
#           (C) 2008, 2009 Róman Joost, Ulf-D. Ehlert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os
import getopt
import re
import xml.sax
import logging

# Configure logging
logging.basicConfig(format="%(message)s", level=logging.WARNING)
Logger = logging.getLogger()

# Nodes containing filereferences in a DocBook XML file
IMAGE_NODES = ["imagedata", "graphic", "inlinegraphic"]

# Regular expression for image files to be checked (png and jpg files only)
IMAGEFILE_REGEX = re.compile('[\w.+-]*\.(png|jpg)$')
# Regular expression for image files to be skipped
IGNORE_IMAGE_REGEX = re.compile('callout')


class FileNameContainer(object):
    """A special purpose container base class.

    This class stores filenames and provides some basic
    access methods common to both derived List classes.

    """
    def __init__(self):
        self.data = {}

    def __contains__(self, key):
        """Is there an entry 'key'?."""
        return self.data.has_key(key)

    def __setitem__(self, key, val):
        """Add a (key, value) pair to the container, e.g.
           filename and some info corresponding to this filename.
        """
        self.data[key] = val

    def add(self, key, val=True):
        """Add a (key, value) pair to the container, e.g.
           filename and some info corresponding to this filename.
        """
        self.data[key] = val

    def clear(self):
        """Remove all entries, i.e. remove all (key, value) pairs."""
        self.data = {}

    def erase(self, key):
        """Mark an entry as invalid.

        Set the info to False rather than deleting it,
        so that "removing" while iterating is possible.
        """
        self.data[key] = False

    def files(self):
        return [x for x in self.data if self.data[x]]

    def size(self):
        """Return the number of non-empty data entries."""
        return len(self.files())

    def __len__(self):
        """Return the number of data entries."""
        return len(self.data)

    def sort_out_valid(self, other):
        """Same as 'difference(self,other)'. When common entries
           have been removed, the remaining filenames are just
           the orphaned files, or the broken links, respectively.
        """
        self.difference(other)

    def difference(self, other):
        """Remove entries common to 'self' and 'other'."""
        for key in (k for k in self.data if k in other):
            self.erase(key)


class ImageFilesList(FileNameContainer):
    """A container for image file names.

    This class is used to collect and save all image files
    in the specified 'images/' directory.

    """
    def __init__(self):
        super(ImageFilesList, self).__init__()
        self.imageroot = ""

    def find(self, imageroot):
        """Search for PNG and JPG files in the image directory."""

        self.clear()
        self.imageroot = os.path.normpath(imageroot)

        Logger.info("\nsearching images in '%s' ..." % self.imageroot)

        for root, dirs, files in os.walk(self.imageroot):
            for prune in [ 'callouts', '.git' ]:
                if prune in dirs:
                    dirs.remove(prune)

            # don't save the prefix common to all entries
            prefixlen = len(self.imageroot) + 1  # path + "/"
            # don't care about other files than images files
            for filename in (name for name in files
                                  if IMAGEFILE_REGEX.match(name)):
                filepath = os.path.join(root, filename)
                Logger.debug(filepath)
                self.add(filepath[prefixlen:])

        Logger.info("found %d image files\n" % len(self))

    def report(self):
        """Print the list of orphaned image files, i.e. image files
           which are not referred to in any XML source file.
        """
        files = self.files()
        Logger.info("%d orphaned image file%s%s" % \
                    (len(files), char_if("s", len(files) != 1),
                                 char_if(":", len(files) != 0)))
        for imagefile in sorted(files):
            print "ORPHANED:", os.path.join(self.imageroot, imagefile)


class ImageReferencesList(FileNameContainer):
    """A container for image file references.

    This class is used to collect and save all image file
    references in the XML source files, i.e. it saves
    ('image-file', 'source-file') pairs.

    """
    def __init__(self):
        super(ImageReferencesList, self).__init__()
        self.cur_files = []	# stack for files in progress
        self.all_files = 0	# visited files
        self.handler   = XMLHandler(self)
        self.parser    = self.make_parser()

    def make_parser(self):
        """Create and return an initialized SAX XMLReader object,
           i.e. an XML parser. A content handler is attached to
           the parser and some appropriate features are set.
        """
        parser = xml.sax.make_parser()
        parser.setContentHandler(self.handler)
        # "Optionally do not perform Namespace processing
        # (implies namespace-prefixes)."
        parser.setFeature(xml.sax.handler.feature_namespaces, 0)
        # "Do not include external general (text) entities."
        parser.setFeature(xml.sax.handler.feature_external_ges, 0)
        # "Do not include any external parameter entities,
        # even the external DTD subset."
        parser.setFeature(xml.sax.handler.feature_external_pes, 0)
        return parser

    def find(self, source_file):
        """Parse XML files and extract image references."""

        Logger.info("parsing XML files ... ")
        self.push_file(source_file)
        Logger.debug("parsing %s" % source_file)

        try:
            self.parser.parse(source_file)
        except xml.sax.SAXException, err:
            Logger.error("ERROR parsing %s" % err)
            sys.exit(65)
        except:
            Logger.error("ERROR reading %s" % source_file)
            sys.exit(65)

        assert(len(self.cur_files) == 1)
        Logger.info("found %d references in %d files" % \
                    (len(self), self.all_files))

    def add(self, imagefile_reference):
        """Add a (reference, location) pair to the list of image files.

        "imagefile_reference" is the value of the 'fileref' attribute
        as returned from the xml-parser. The common prefix "images/"
        will be removed.
        "location" is the name of the current xml source file.
        """
        key = imagefile_reference.replace("images/", "")
        val = self.current_file()
        super(ImageReferencesList, self).add(key, val)

    def report(self):
        """Print the list of broken image referencess
           in the XML source file(s).
        """
        files = self.files()
        Logger.info("%d broken image reference%s%s" % \
                    (len(files), char_if("s", len(files) != 1),
                                 char_if(":", len(files) != 0)))
        for imagefile in sorted(files):
            print "BROKEN: images/%s IN %s" % (imagefile, self.data[imagefile])

    # Internal stack methods to keep track of the opened files

    def current_file(self):
        """The file currently parsed."""
        return self.cur_files[-1]	# top of filenames stack

    def push_file(self, filename):
        """Add entry to internal stack of filenames."""
        self.cur_files.append(filename)
        self.all_files += 1

    def pop_file(self):
        """Remove top entry from internal stack of filenames."""
        return self.cur_files.pop()


class XMLHandler(xml.sax.handler.ContentHandler):
    """A content handler class as defined by the SAX API."""
    def __init__(self, owner):
        #super(XMLHandler, self).__init__()
        self.owner = owner

    def startElement(self, name, attrs):
        """Handle image nodes."""
        if name in IMAGE_NODES:
            try:
                fileref = attrs.getValue('fileref')
                if not IGNORE_IMAGE_REGEX.match(fileref):
                    self.owner.add(fileref)
            except KeyError:
                self.error("missing 'fileref' attribute")
        if name == "xi:include" and attrs.has_key('href'):
            filename = os.path.join(os.path.dirname(self.owner.current_file()),
                                    attrs.getValue('href'))
            if os.path.isfile(filename):
                parser = self.owner.make_parser()
                self.owner.push_file(filename)
                Logger.debug("parsing %s" % filename)
                parser.parse(filename)
                self.owner.pop_file()
            else:
                self.error("missing file: '%s'" % filename)

    def error(self, errmsg=""):
        """Wrapper function for raising an xml.sax.SAXException.

        This is a simple workaround to raise an exception with
        line and column number provided.
        """
        try:
            line = self._locator.getLineNumber()
            column = self._locator.getColumnNumber()
        except AttributeError:
            line, column = '?', '?'
        errmsg = "%s:%s:%s: %s" % \
                 (self.owner.current_file(), line, column, errmsg)
        raise xml.sax.SAXException(errmsg)


def main():
    """The main program (hmm, what did you expect?).

       The algorithm for validating image files and references is
       very simple:

       Let
           (1) R := (set of) all image file references,
           (2) I := (set of) all image files,
       then
           (3) B := R \ I = R \ (R ∩ I)
       is the set containing files in R but not in I, that is the set
       of broken references,
           (4) O := I \ R = I \ (R ∩ I)
       is the set containing files in I but not in R, that is the set
       of images not referenced in the XML files (orphaned images).
    """
    gimp_help_root_dir     = "."
    xml_dir                = "src"
    img_dirs               = "images/C"
    languages              = "C"
    xml_root_file          = "gimp.xml"
    find_orphaned_images   = False
    find_broken_references = False

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hvr:x:s:blof:",
                         ["help", "verbose",
                          "root=", "xmldir=", "srcdir=", "file=",
                          "broken", "links", "orphaned",])
    except getopt.GetoptError, err:
        usage(64, str(err))

    for opt, arg in opts:
        if opt == "-h" or opt == "--help":
            usage()
        elif opt == "-v" or opt == "--verbose":
            if not Logger.isEnabledFor(logging.DEBUG):
                Logger.setLevel(Logger.getEffectiveLevel() - logging.DEBUG)
        elif opt in ["-r", "--root"]:
            gimp_help_root_dir = arg
        elif opt in ["-x", "--xmldir", "-s", "--srcdir"]:
            xml_dir = arg
            xml_root_file = os.path.join(xml_dir, xml_root_file)
        elif opt in ["-b", "--broken", "-l", "--links"]:
            find_broken_references = True
        elif opt in ["-o", "--orphaned"]:
            find_orphaned_images = True
        elif opt == "-f" or opt == "--file":
            find_broken_references = True
            xml_root_file = arg

    languages = args or ["C"];
    if "en" in languages:
        if "C" in languages:
            languages.remove("en")
        else:
            languages[languages.index("en")] = "C"

    # Change to user specified root dir.
    if gimp_help_root_dir != ".":
        try:
            os.chdir(gimp_help_root_dir)
        except OSError, (errno, strerror):
            Logger.error("ERROR: %s: %s" % (strerror, gimp_help_root_dir))
            sys.exit(errno)

    # Check for the correct directory.
    if not (os.path.isdir("images/") and os.path.isdir("src/")):
        usage(66, "This script must be called from the " +
                   "gimp-help-2 root directory.")

    # We need an existing xml source file to parse.
    if not os.path.isfile(xml_root_file):
        if os.path.isfile(os.path.join(xml_dir, xml_root_file)):
            xml_root_file = os.path.join(xml_dir, xml_root_file)
        else:
            usage(66, "Cannot find " + xml_root_file + ".")

    # When finding orphaned images, we must parse all xml files.
    #if find_orphaned_images and (xml_root_file != "src/gimp.xml"):
    #    usage(64, "'--file <file>' and '--orphaned' are mutually exclusive.")

    # If no action specified: search for broken image references.
    if not (find_orphaned_images or find_broken_references):
        find_broken_references = True

    image_refs  = ImageReferencesList()
    image_files = ImageFilesList()

    # find all image references.
    image_refs.find(xml_root_file)                         # (1)

    # find all image files in "images/C", then
    # remove intersection of image references and image files;
    # the result is the list of invalid (broken) references.
    if find_broken_references:
        image_files.find("images/C")                       # (2)
        image_refs.sort_out_valid(image_files)             # (3)
        image_refs.report()

    # find all image files in "images/LANG", then
    # remove intersection of image references and image files;
    # the result is the list of orphaned image files.
    if find_orphaned_images:
        img_dirs = ("images/" + lang for lang in languages)
        for imgdir in img_dirs:
            # if possible, avoid searching in "images/C" twice
            if not (imgdir == image_files.imageroot):
                image_files.find(imgdir)                   # (2)
            image_files.sort_out_valid(image_refs)         # (4)
            image_files.report()


def usage(exitcode=0, msg=""):
    """Help the user."""
    if exitcode > 0 or msg:
        sys.stderr.write("Error: %s\n" % msg)
        sys.stderr.write("(try \"%s --help\")\n" % sys.argv[0])
    else:
        sys.stdout.write ( """\
validate_references - validates image file references in DocBook xml files

Copyright (C) 2006-2007 Róman Joost,
          (C) 2008-2012 Róman Joost, Ulf-D. Ehlert

Usage: validate_references.py [options] [language ...]

    options:
        -h | --help          this help
        -v | --verbose       verbose; may be provided twice (-vv)
        -r | --root <dir>    specify the gimp-help-2 root directory
        -x | --xmldir <dir>  specify the XML files directory
        -o | --orphaned      check for orphaned image files
        -b | --broken        check for broken links (default action)
\n""")
    sys.exit(exitcode)


def char_if(char, cond):
    if cond:
        return char
    else:
        return ""


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# This file is part of the gimp-help-2 project and is
# Copyright 2004, 2005, 2006, 2007 Roman Joost
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
import sys
import os
import os.path
import getopt
import re
import xml.xpath
import xml.dom.minidom

ALL_LINGUAS = re.compile('ALL_LINGUAS="(\w.+)\w')


class GIMPHelpXMLParser(object):
    """Parses the XML sources..."""

    def __init__(self, filepath, xpathexpr="//help-item[@id]"):
        self.ids = []
        assert filepath != ""
        self.filepath = filepath
        self.xpathexpr = xpathexpr

    def get_ids(self):
        """Returns a list of ids parsed from gimp-help.xml."""
        fh = open(self.filepath, 'r')
        dom = xml.dom.minidom.parse(fh)
        print "Parsing XML file: %s" % os.path.basename(os.path.dirname(self.filepath))
        # XXX the ids
        ids = xml.xpath.Evaluate(self.xpathexpr, dom)
        self.ids = [x.getAttribute('id') for x in ids ]
        return self.ids


class GIMPHelpHeaderParser(object):
    """Parses gimphelp-ids.h and indexes the ids"""

    helpid = re.compile('gimp-.[a-z-]+')

    def __init__(self, filepath):
        assert filepath != ""
        self.filepath = filepath
        self.ids = []

    def parse(self):
        h_file = open(self.filepath, "r")
        # XXX do a read() and match with a regexp
        for line in h_file.readlines():
            try:
                str = self.helpid.search(line).group()
                self.ids.append(str)
            except AttributeError:
                pass


class Statistics(object):
    """Creates statistics output."""
    
    def __init__(self, headerpath, helproot):
        self.helproot = helproot
        self.hp = GIMPHelpHeaderParser(headerpath)
        self.hp.parse()
        self.totals = len(self.hp.ids)
        self.docs = self.getDocuments(helproot)
        self.statistics = self._generateStatistics()

    def getDocuments(self, helproot):
        """ returns a dictionary with languages as keys and paths to the
            xml documents as values 
        """
        linguas = get_linguas(helproot) 
        result = {}
        for lang in linguas:
            # puzzling the path to the gimp-help.xml file together
            helpfile = os.path.join(helproot, 'html', lang,
                                    'gimp-help.xml')
            if os.path.exists(helpfile):
                xp = GIMPHelpXMLParser(helpfile)
                result[lang] = xp.get_ids()

        return result

    def get_ids_from_lang(self, lang):
        """Iterator over ids of given language."""
        if lang not in self.docs.keys():
            raise KeyError("%s not found in indexed documents." % lang)
        for id in self.docs[lang]:
            yield id

    def _generateStatistics(self):
        """ generates statistics for every language
            returns list with dicts
                dicts = {written items, todo items, percentage written,
                language}
        """
        result = []
        helpids = self.hp.ids

        for lang in self.docs.keys():
            other_ids = []
            matched = []

            for id in self.get_ids_from_lang(lang):
                if id not in helpids:
                    matched.append(id)
                else:
                    other_ids.append(id)

            done = len(matched)
            add = len(other_ids)
            if done < self.totals:
                todo = self.totals - done
                prc_done = done*100/self.totals
            else:
                todo = 0
                prc_done = 100

            lang = self._makedict(done=done,
                                 todo=todo,
                                 others=add,
                                 prc_done=prc_done,
                                 lang=lang)
            result.append(lang)

        return result

    def getInvalidIds(self):
        """ parses one gimp-help.xml file and compare the id's found
            with help ids. If the id from the xml file is not in help
            ids, append it to invalid.
            returns list with invalid ids (str)
        """
        if not self.docs:
            raise ValueError("Can't parse a gimp-help.xml for any"
                             " Language. Maybe you need to build the HTML"
                             " docs? Check if gimp-help.xml files exist.")

        for lang in self.docs.keys():
            invalid = []
            skip = ['faq', 'using', 'fdl', 'glossary',
                'introduction', 'gimp-main', 'legal', 'tools-color',
                'tools-paint', 'tools-selection', 'tools-transform',
                'tools-menu', 'plug-in']

            for id in self.get_ids_from_lang(lang):
                not_invalid = None
                if id in skip:
                    continue

                not_invalid = filter(lambda k, y=0:\
                    y + self._is_substring(id, k), skip)

                if not_invalid:
                    continue

                if not self._is_helpid(id):
                    invalid.append(id)

            invalid.sort()
        return invalid


    def getInvalidFilenames(self):
        """ indexes all filenames and compare it with the help ids
            if filename is not in help ids append it to invalid list
            returns list with invalid filenames(str)
        """
        # XXX needs to be rewritten
        invalid = []
        names = {}
        root = os.path.join(self.helproot, "src")
        # dirs which we can savely skip
        filter_directories = ['glossary', 'faq', 'filters']

        # filenames which we can savely skip
        filter_files = ['gimp', 'introduction', 'menu']

        for root, dirs, files in os.walk(root):
            if 'CVS' in dirs:
                dirs.remove('CVS') # don't visit CVS dirs

            for dir in dirs:
                if dir in filter_directories:
                    dirs.remove(dir)

            for file in files:
                if file.endswith('xml'):
                    id = file[:-4]
                    names[id] = os.path.join(root, file)

        # remove the valid entries
        for id in names.keys():
            not_invalid = 0

            not_invalid = filter(lambda k, y=0:\
                y + self._is_substring(id, k), filter_files)
            if not_invalid:
                continue

            if self._is_helpid('gimp-' + id):
                del names[id]

        return names

    def printHTMLStatistics(self):
        """ returns HTML Code with statistics """
        raise NotImplementedError 

    def printTextStatistics(self, print_invalid=0):
        """ prints Text Statistics """
        invalid_ids = self.getInvalidIds()
        invalid_files = self.getInvalidFilenames()

        print "General statistics:"
        print "==================="

        for dict in self.statistics:
            done = '#'*int(dict['prc_done']/10)
            todo = ' '*(10-(dict['prc_done']/10))
            print "\nLanguage: %s" %dict['lang']
            print "Found titles for: %i ids - Todo: %s ids - other ids: %s"\
                %(dict['done'], dict['todo'], dict['others'])
            print "Percent done: |%s%s| %s%%" %(done,todo, dict['prc_done'])

        print "\n\nInvalid ids:"
        print "================"
        print "Found %i ids which are not part of gimphelp-ids.h" %len(invalid_ids)

        if print_invalid:
            invalid_ids.reverse()
            for id in invalid_ids:
                print "%s" %id
        else:
            print "Hint: start the program again with parameter >>-i<<",
            print "if you want to view them."

        print "\n\nInvalid filenames:"
        print "======================"
        print "Found %i filenames which are not part of gimphelp-ids.h" %len(invalid_files)

        if print_invalid:
            keys = invalid_files.keys()
            keys.sort()
            for k in keys:
                print "%s -> %s" %(k, invalid_files[k])
        else:
            print "Hint: start the program again with parameter >>-i<<",
            print "if you want to view them."

    def _makedict(self, **kwargs):
        return kwargs

    def _is_substring(self, str, substr):
        """ returns True if str is substr from str """
        return str.find(substr) >= 0

    def _is_helpid(self, name):
        """ returns True if the file is in helpids otherwise 0 """
        return name is not None and name in self.hp.ids


def get_linguas(gimphelppath):
    """ returns the linguas set in configure.in in gimp-help-2 dir """
    gimphelppath = os.path.join(gimphelppath, "html")
    if os.environ.has_key('ALL_LINGUAS'):
        linguas = os.environ.get('ALL_LINGUAS')
        linguas = linguas.split(' ')
        return linguas 

    linguas = os.listdir(gimphelppath)
    linguas.remove('images')
    return linguas


def main():
    header = None
    xml = None
    helppath = "."
    print_invalid = 0
    gimpheaderfile = "gimphelp-ids.h"
    xmldocs = {}

    try:
        opts, args = getopt.getopt(sys.argv[1:], "hx:g:i")
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    if not opts:
        errormsg = "Please set the path to gimp-help-2 and gimp installation."
        usage(errormsg)
        sys.exit(1)

    for o, a in opts:
        if o == "-h":
            usage()
        if o == "-x":
            helppath = a
        if o == "-g":
            gimp = os.path.join(a, "app", "widgets", gimpheaderfile)
        if o == "-i":
            print_invalid = 1

    gimp = os.path.abspath(gimp)
    helppath = os.path.abspath(helppath)
    if os.path.exists(gimp) and os.path.exists(helppath):
        st = Statistics(gimp, helppath)
        st.printTextStatistics(print_invalid)
        sys.exit(1)
    else:
        usage()
        print "Error: one of your path is invalid!"
        sys.exit(0)


def usage(errormsg=None):
    
    print """idlookup.py - Copyright 2004 Roman Joost (gimp-help-2)
generates some statistical information about the documentation process 

usage: idlookup.py [options]

    options:
        -g      path to the GIMP sources (eg. /opt/gimp)
        -x      path to the gimp-help-2 sources (eg. /opt/gimp-help-2)
        -i      print ids which are supposed to be invalid
        -h      this help"""
    if errormsg:
        print "\nError: %s" % errormsg

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Copyright (c) 2004, 2005, 2006 Danilo Šegan <danilo@gnome.org>.
# Copyright (c) 2009 Claude Paroz <claude@2xlibre.net>.
#
# This file is part of xml2po.
#
# xml2po is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# xml2po is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with xml2po; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

# xml2po -- translate XML documents
VERSION = "0.18.0 (patched by GIMP Documentation Team)"

# Versioning system (I use this for a long time, so lets explain it to
# those Linux-versioning-scheme addicts):
#   1.0.* are unstable, development versions
#   1.1 will be first stable release (release 1), and 1.1.* bugfix releases
#   2.0.* will be unstable-feature-development stage (milestone 1)
#   2.1.* unstable development betas (milestone 2)
#   2.2 second stable release (release 2), and 2.2.* bugfix releases
#   ...
#
import sys
import os
import getopt
import tempfile

NULL_STRING = '/dev/null'
if not os.path.exists('/dev/null'): NULL_STRING = 'NUL'

def usage (with_help = False):
    print >> sys.stderr, "Usage:  %s [OPTIONS] [XMLFILE]..." % (sys.argv[0])
    if with_help:
        print >> sys.stderr, """
OPTIONS may be some of:
    -a    --automatic-tags     Automatically decides if tags are to be considered
                                 "final" or not
    -k    --keep-entities      Don't expand entities
    -e    --expand-all-entities  Expand ALL entities (including SYSTEM ones)
    -m    --mode=TYPE          Treat tags as type TYPE (default: docbook)
    -o    --output=FILE        Print resulting text (XML or POT) to FILE
    -p    --po-file=FILE       Specify PO file containing translation, and merge
    -r    --reuse=FILE         Specify translated XML file with the same structure
    -t    --translation=FILE   Specify MO file containing translation, and merge
    -u    --update-translation=LANG.po   Updates a PO file using msgmerge program

    -l    --language=LANG      Set language of the translation to LANG
          --mark-untranslated  Set 'xml:lang="C"' on untranslated tags

    -v    --version            Output version of the xml2po program

    -h    --help               Output this message

EXAMPLES:
    To create a POTemplate book.pot from input files chapter1.xml and
    chapter2.xml, run the following:
        %(command)s -o book.pot chapter1.xml chapter2.xml

    After translating book.pot into de.po, merge the translations back,
    using -p option for each XML file:
        %(command)s -p de.po chapter1.xml > chapter1.de.xml
        %(command)s -p de.po chapter2.xml > chapter2.de.xml
""" % {'command': sys.argv[0]}


def main(argv):
    if not argv:
        usage()
        sys.exit(2)

    name = os.path.join(os.path.dirname(__file__), '..')
    if not name in sys.path:
        sys.path.insert(0, name)

    from xml2po import Main

    # Default parameters
    default_mode = 'docbook'
    operation = 'pot' # 'pot', 'merge', 'update'
    output  = '-' # this means to stdout
    options = {
        'mark_untranslated'   : False,
        'expand_entities'     : True,
        'expand_all_entities' : False,
    }
    origxml = ''
    mofile = None
    mofile_tmppath = None

    try: opts, remaining_args = getopt.getopt(argv, 'avhkem:t:o:p:u:r:l:',
                               ['automatic-tags','version', 'help', 'keep-entities', 'expand-all-entities', 'mode=', 'translation=',
                                'output=', 'po-file=', 'update-translation=', 'reuse=', 'language=', 'mark-untranslated' ])
    except getopt.GetoptError:
        usage(True)
        sys.exit(2)

    for opt, arg in opts:
        if opt in ('-m', '--mode'):
            default_mode = arg
        if opt in ('-a', '--automatic-tags'):
            default_mode = 'basic'
        elif opt in ('-k', '--keep-entities'):
            options['expand_entities'] = False
        elif opt in ('--mark-untranslated',):
            options['mark_untranslated'] = True
        elif opt in ('-e', '--expand-all-entities'):
            options['expand_all_entities'] = True
        elif opt in ('-l', '--language'):
            options['translationlanguage'] = arg
        elif opt in ('-t', '--translation'):
            mofile = arg
            operation = 'merge'
            if 'translationlanguage' not in options:
                options['translationlanguage'] = os.path.split(os.path.splitext(mofile)[0])[1]
        elif opt in ('-r', '--reuse'):
            origxml = arg
        elif opt in ('-u', '--update-translation'):
            operation = 'update'
            po_to_update = arg
        elif opt in ('-p', '--po-file'):
            mofile_handle, mofile_tmppath = tempfile.mkstemp()
            os.close(mofile_handle)
            pofile = arg
            operation = 'merge'
            if 'translationlanguage' not in options:
                options['translationlanguage'] = os.path.split(os.path.splitext(pofile)[0])[1]
            os.system("msgfmt -o %s %s >%s" % (mofile_tmppath, pofile, NULL_STRING)) and sys.exit(7)
            mofile = mofile_tmppath
        elif opt in ('-o', '--output'):
            output = arg
        elif opt in ('-v', '--version'):
            print VERSION
            sys.exit(0)
        elif opt in ('-h', '--help'):
            usage(True)
            sys.exit(0)

    if operation == 'update' and output != "-":
        print >> sys.stderr, "Option '-o' is not yet supported when updating translations directly. Ignoring this option."

    # Treat remaining arguments as XML files
    filenames = []
    while remaining_args:
        filenames.append(remaining_args.pop())

    try:
        xml2po_main = Main(default_mode, operation, output, options)
    except IOError:
        print >> sys.stderr, "Error: cannot open file %s for writing." % (output)
        sys.exit(5)

    if operation == 'merge':
        if len(filenames) > 1:
            print  >> sys.stderr, "Error: You can merge translations with only one XML file at a time."
            sys.exit(2)

        if not mofile:
            print >> sys.stderr, "Error: You must specify MO file when merging translations."
            sys.exit(3)

        xml2po_main.merge(mofile, filenames[0])

    elif operation == 'update':
        xml2po_main.update(filenames, po_to_update)

    elif origxml:
        xml2po_main.reuse(origxml, filenames[0])

    else:
        # Standard POT producing
        xml2po_main.to_pot(filenames)

    if mofile_tmppath:
        os.remove(mofile_tmppath)

# Main program start
if __name__ == '__main__':
    main(sys.argv[1:])
else:
    raise NotImplementedError
